<?php

namespace Ardent\Undercurrent\Kernel;

use Ardent\Undercurrent\Config\AppConfig;
use Ardent\Undercurrent\Config\GenericConfig;
use Ardent\Undercurrent\Console\CommandsConfig;
use Ardent\Undercurrent\Container\ContainerInterface;
use Ardent\Undercurrent\Container\GenericContainer;
use Ardent\Undercurrent\Http\GenericRouter;
use Ardent\Undercurrent\Http\RoutesConfig;
use Ardent\Undercurrent\Http\RouterInterface;
use Ardent\Undercurrent\Logger\LogContainer;
use Ardent\Undercurrent\Logger\LoggerInterface;
use Ardent\Undercurrent\View\ViewHelper;
use Doctrine\DBAL\DriverManager;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\ORMSetup;

class BaseKernel
{
    public function __construct(
        private readonly AppConfig $appConfig,
    )
    {
    }

    public function setup(): ContainerInterface
    {
        $container = (new GenericContainer());
        $container
            ->alias(RouterInterface::class, GenericRouter::class)
            ->alias(ContainerInterface::class, GenericContainer::class)
            ->alias(LoggerInterface::class, LogContainer::class)
            ->add(GenericContainer::class, fn($container) => $container)
            ->add(AppConfig::class, fn() => $this->appConfig)
            ->add(GenericRouter::class)
            ->add(ViewHelper::class)
            ->add(EntityManager::class, function (ContainerInterface $container) {
                $config = $container->get(GenericConfig::class);
                $paths = [$container->get(AppConfig::class)->getEntityPath()];

                $ormConfig = ORMSetup::createAttributeMetadataConfiguration($paths, $config->get('dev'));
                $connection = DriverManager::getConnection($config->get('orm'), $ormConfig);

                return new EntityManager($connection, $ormConfig);
            })
            ->add(LogContainer::class);

        // App related dependencies
        $this->dependencies($container);
        $configPath = $this->appConfig->getRootPath() . '/config.php';
        if (file_exists($configPath)) {
            $container->add(GenericConfig::class, fn() => include $configPath);
        }

        return $container;
    }

    protected function addControllers(ContainerInterface $container, RoutesConfig $routes): void
    {
        foreach ($routes->getControllers() as $controller) {
            $container->add($controller);
        }

        $container->add(RoutesConfig::class, fn() => $routes);
    }

    protected function addCommands(ContainerInterface $container, CommandsConfig $commands): void
    {
        foreach ($commands->getCommands() as [$commandConfig, $commandClass]) {
            $container->add($commandClass);
        }

        $container->add(CommandsConfig::class, fn() => $commands);
    }

    protected function dependencies(ContainerInterface $container): void
    {
    }
}